/* CouchDB C API definition
 *
 * CouchDB C API (libcouchdb) - Copyright (c) 2006 Dirk Schalge
 *
 * Thanks to Damien Katz and all other involved developers for this great
 * database!
 *
 * See doc/LICENSE for licensing details.
 */

#ifndef CDB_HEADER_API
#define CDB_HEADER_API

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <stdarg.h>
#include <curl/curl.h>

#include "list.h"
#include "struct.h"

/* Welcome to the show
 *
 * CouchDB C API
 *
 */


/* Set connection settings for the couchdb.
 * This function must be called before any other operation is allowed.
 */
extern Couch *couch_init ( char *server, char *db );

/* When you're done call this function to clear the memory. */
extern void couch_cleanup ( Couch **c );

/* Select database for further actions. */
extern int couch_db_use ( Couch *c, char *db );



/* Save a document to the database. Returns the DocumentID. */
extern int couch_doc_save ( Couch *c, CouchDocument *d );

/* Removes an existing document. */
extern int couch_doc_remove ( Couch *c, char *docid );

/* Create a new Document. Rerturns a Document structure. */
extern CouchDocument *couch_doc_create ( char *name );

/* Get an existing Document. Returns a Document structure. */
extern CouchDocument *couch_doc_get ( Couch *c, char *docid );

/* Get a list of (CouchDocument *) with all available Documents */
extern List *couch_doc_get_all ( Couch *c );

/* Set field values in a Document. */
extern void couch_doc_set ( CouchDocument *d, char *name, ... );

/* When you don't need to edit the document anymore, clear the memory with this function. */
extern void couch_doc_cleanup ( CouchDocument **d );

/* Clear list of documents (that was created by couch_get_doc_all()) */
extern void couch_doc_list_cleanup ( List **l );




/* Performs a table, the result is a list of (CouchDocument *) */
extern List *couch_table_compute ( Couch *c, char *name );


/* Creates a database. The database stored in c is ignored. */
extern int couch_db_create ( Couch *c, char *name );

/* Deletes a database. The database stored in c is ignored. */
extern int couch_db_remove ( Couch *c, char *name );

/* Gets a list of (char *) with all available database names. */
extern List *couch_db_get_all ( Couch *c );

#endif
