%% CouchDb
%% Copyright (C) 2006  Damien Katz
%% 
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%% 
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%% 
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

-module(couch_doc).

-export([new/0,new/1,set_field/3,set_fields/2,get_field/2,add_rev/2]).
-export([get_field/3,map/2,get_table_queries/1]).

-include("couch_db.hrl").
-include_lib("xmerl/include/xmerl.hrl").
	
new() ->
	#doc{}.
	
new(FieldNameValueList) ->
	set_fields(#doc{}, FieldNameValueList).
	
set_field(Doc, FieldName, Value) ->
	SummaryFields = dict:store(FieldName, Value, Doc#doc.summary_fields),
	Doc#doc{summary_fields = SummaryFields}.

set_fields(Doc, []) ->
	Doc;
set_fields(Doc, [{FieldName, Value} | Rest]) ->
	Doc2 = set_field(Doc, FieldName, Value),
	set_fields(Doc2, Rest).
	
	
get_field(Doc, FieldName) ->
	dict:find(FieldName, Doc#doc.summary_fields).
	
get_field(Doc, FieldName, DefaultValue) ->
	case dict:find(FieldName, Doc#doc.summary_fields) of
	{ok, Value} ->
		Value;
	error ->
		DefaultValue
	end.
	
add_rev(Doc, RevId) ->
	Doc#doc{revisions = [RevId | Doc#doc.revisions]}.

get_table_queries(Doc) ->
	dict:fold(
		fun(FieldName, [Value], Tables)->
				case FieldName of 
				"$table_" ++ TableName ->
					[{TableName, Value} | Tables];
				_Else ->
					Tables
				end
		end,
		[], Doc#doc.summary_fields).

map(Doc, Fun) ->
	FoldFun = fun(Key, Value, Acc) -> [Fun(Key,Value) | Acc] end,
	lists:reverse(dict:fold(FoldFun, [], Doc#doc.summary_fields)).
	

	