/*
Fabric formula engine
Copyright (C) 2006  Damien Katz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef FABRIC_ICU_H
#define FABRIC_ICU_H

#include "unicode/ucasemap.h"
#include "unicode/uchar.h"
#include "unicode/unistr.h"
#include "unicode/ustring.h"
#include "unicode/regex.h"


#define CHECK_STATUS(st) if (U_FAILURE(status)) {throw RuntimeException("ICU", "Generic ICU Error");}

template<class T> void utf8_convert(const char* pIn, int inLen, T& retBuffer)
{
	UErrorCode status = U_ZERO_ERROR;
	retBuffer.resize(inLen);
	int resultLen = 0;
	u_strFromUTF8(&retBuffer[0], inLen, &resultLen, pIn, inLen, &status);
	retBuffer.resize(resultLen); //shrink/grow the buffer to the correct length

	if (status == U_BUFFER_OVERFLOW_ERROR) {
		// we were'nt successful because the buffer was too small,
		// try again with the properly sized buffer
		status = U_ZERO_ERROR;
		u_strFromUTF8(&retBuffer[0], resultLen, &resultLen, pIn, inLen, &status);
	}

	CHECK_STATUS(status);
}

template<class T> void utf8_convert(const TextElement& text, T& retBuffer)
{
	utf8_convert(text.Ptr(), text.LenInt(), retBuffer);
}


#endif // FABRIC_ICU_H
