/* CouchDB database-handling functions
 *
 * CouchDB C API (libcouchdb) - Copyright (c) 2006 Dirk Schalge
 *
 * Thanks to Damien Katz and all other involved developers for this great
 * database!
 *
 * See doc/LICENSE for licensing details.
 */

#include "libcouchdb.h"
#include "curl.h"
#include "xml.h"

int couch_db_create ( Couch *c, char *name )
{
	if ( !c || !name )
	{
		errno = EINVAL;
		return -1;
	}

	/* send PUT */
	couch_curl_db_put ( c, name );

	/* parse resposne */
	return couch_xml_parse_response ( c );
}

int couch_db_remove ( Couch *c, char *name )
{
	if ( !c || !name )
	{
		errno = EINVAL;
		return -1;
	}
	/* send DELETE */
	couch_curl_db_delete ( c, name );

	/* parse response */
	return couch_xml_parse_response ( c );
}

List *couch_db_get_all ( Couch *c )
{
	List *l = NULL;

	if ( !c )
	{
		errno = EINVAL;
		return NULL;
	}

	/* send GET */
	couch_curl_db_get_all ( c );

	l = couch_xml_parse_db_get_all ( c->xml );

	if ( l )
	{
		c->result->type = 1;
		return l;
	}
	else
	{
		/* parse error */
		couch_xml_parse_response ( c );
		return NULL;
	}
}
