dojo.require("dojo.widget.Tree");
dojo.require("dojo.widget.TreeSelector");
dojo.require("dojo.widget.ContentPane");
dojo.require("dojo.widget.LayoutContainer");
dojo.require("dojo.widget.FloatingPane");
dojo.require("dojo.io.*");
dojo.require("dojo.event.*");
dojo.require("dojo.widget.Button");

dojo.addOnLoad(init);

var COUCHDBPATH="/";
var selectedDb="";

function init(){
	dojo.debug("Ready to run script...");
	dojo.byId("ly_Content").innerHTML=dojo.byId("WelcomeContent").innerHTML
	buildDbNav();
}

function buildDbNav(){
	dojo.byId("ly_Nav").innerHTML=""; /*Clear nav area*/
	selectedDb="";
	clearButtons(["buttonCreateDb"]);
	
	var tree = dojo.widget.createWidget("Tree", {toggle:"fade", selector:"treeSelector", id:"navTree"});
	dojo.byId("ly_Nav").appendChild(tree.domNode);
	var rootNode = dojo.widget.createWidget("TreeNode",{title:"CouchDb", expandLevel:"1"});
	tree.addChild(rootNode);
	dojo.io.bind({
		url:COUCHDBPATH+"$all_dbs",
		load:function(type, data, event){
			dojo.debug(dojo.dom.innerXML(data));
			var db = data.getElementsByTagName("db");
			for(var i=0; i<db.length; i++){
				dojo.debug(i + ")text=" + dojo.dom.textContent(db[i]));
				rootNode.addChild(dojo.widget.createWidget("TreeNode",{title:(dojo.dom.textContent(db[i]))}) );
			}
		},
		error:function(type, data, event){alert(type+":"+data.message)},
		transport:"XMLHTTPTransport",
		mimetype:"text/xml"
	});
	
	dojo.event.topic.subscribe('navTree/titleClick',
		function(menuItem){
			if(menuItem.source.depth==0){
				selectedDb="";
				dojo.byId("SelectedDbTitle").innerHTML="";
				dojo.byId("ly_Content").innerHTML=dojo.byId("WelcomeContent").innerHTML;
				clearButtons(["buttonCreateDb"]);
			}else{
				selectedDb=menuItem.source.title;
				dojo.debug("selectedDb="+selectedDb);
				dojo.byId("SelectedDbTitle").innerHTML=selectedDb;
				buildAllDocsTable();
			}
		}
	);
}

function buildAllDocsTable(){
	dojo.io.bind({
		url:COUCHDBPATH+selectedDb+"$all_docs",
		load:function(type, data, event){
			var row=data.getElementsByTagName("row");
			if(row.length==0){
				dojo.byId("ly_Content").innerHTML="<h3>No Documents</h3>";
			}else{
				dojo.byId("ly_Content").innerHTML="<h3>"+row.length+" documents</h3><div id='alldoclist'></div>";
				for(var i=0; i<row.length; i++){
					dojo.byId("alldoclist").innerHTML+="<div class='docidlist' onclick='getDocument(\""+row[i].getAttribute("docid")+"\")'>"+row[i].getAttribute("docid")+"</div>";
				}
			}
			clearButtons(["buttonDeleteDb","buttonCreateDoc","buttonCreateTable"]);
		},
		error:function(type, data, event){alert(type+":"+data.message)},
		transport:"XMLHTTPTransport",
		mimetype:"text/xml"
	});
}

function getDocument(docid){
	dojo.io.bind({
		url:COUCHDBPATH+selectedDb+docid,
		load:function(type, data, event){
			dojo.byId("ly_Content").innerHTML="<h3>Document " + docid + "</h3><div id='doccontent'></div>";
			dojo.byId("ly_Content").innerHTML+="<div id='currentDocId' style='display:none'>" + docid + "</div>";
			var field=data.getElementsByTagName("field");
			for(var i=0; i<field.length; i++){
				dojo.byId("doccontent").innerHTML+="<label>"+field[i].getAttribute("name")+":</label>"+dojo.dom.textContent(field[i]) + "<br/>";
				if(field[i].getAttribute("name").indexOf("$table")==0){
					var tmpTable = field[i].getAttribute("name");
					dojo.debug(tmpTable + ":" + tmpTable.substring(7, tmpTable.length))
					buildTable(docid, tmpTable.substring(7, tmpTable.length));
				}
			}
			clearButtons(["buttonBack","buttonViewDocSource","buttonDeleteDoc"]);
		},
		error:function(type, data, event){alert(type+":"+data.message);},
		transport:"XMLHTTPTransport",
		mimetype:"text/xml"
	});
}

function buildTable(docid, tablename){
	dojo.io.bind({
		url:COUCHDBPATH+selectedDb+docid+":"+tablename,
		load:function(type, data, event){
			var table = data.getElementsByTagName("table");
			if(table[0].getAttribute("total_rows") != "0"){
				var tmpHtml = "<div class='tablediv'><h3>"+docid+":"+tablename+"</h3><table>";
				var row = table[0].getElementsByTagName("row");
				for(var i=0; i<row.length; i++){
					var col = row[i].getElementsByTagName("column");
					if(i==0){
						tmpHtml += "<tr>"
						for(var j=0; j<col.length; j++){
							tmpHtml += "<th>" + col[j].getAttribute("name") + "</th>";
						}
						tmpHtml += "</tr>"
					}
					tmpHtml += "<tr>";
					for(var j=0; j<col.length; j++){
						tmpHtml += "<td>" + (dojo.dom.textContent(col[j])) + "</td>";
					}
					tmpHtml += "</tr>";
				}
				tmpHtml += "</table></div>";
				dojo.byId("doccontent").innerHTML+=tmpHtml;
			}
		},
		error:function(type, data, event){alert(type+":"+data.message);},
		transport:"XMLHTTPTransport",
		mimetype:"text/xml"
	});
}

function clearButtons(sb){
	var buttons = ["buttonCreateDb","buttonDeleteDb","buttonDeleteDoc","buttonBack","buttonCreateDoc","buttonSaveNewDoc",
			"buttonCreateTable","buttonSaveNewTable","buttonViewDocSource"];
	for(var i=0; i<buttons.length; i++){dojo.byId(buttons[i]).style.display="none"}
	for(i=0; i<sb.length; i++){dojo.byId(sb[i]).style.display=""}	//show button
}

function deleteDb(){
	var confirmText=confirm("Are you sure you want to DELETE "+selectedDb+" database?");
	if(confirmText){
		dojo.io.bind({
		url:COUCHDBPATH+selectedDb,
		method:"delete",
		load: function(type, data, event){
			if(event.status=="202"){
				dojo.byId("ly_Content").innerHTML="<h3>Database Deleted</h3>";
				buildDbNav();
				dojo.byId("SelectedDbTitle").innerHTML="";
				clearButtons([]);
			}else{
				dojo.byId("ly_Content").innerHTML=event.status+":Failed";
			}
		},
		error: function(type, data, event) {alert(type+":"+data.message)}
		});
	}
}

function createDb(){
	var dbname = prompt("Enter a database name","");
	if(dbname!=""&&dbname!=null){
		dojo.debug("CreateDb=" + COUCHDBPATH+dbname)
		dojo.io.bind({
		url:COUCHDBPATH+dbname,
		method:"put",
		load: function(type, data, event){
			if(event.status=="201"){
				dojo.byId("ly_Content").innerHTML="<h3>"+dbname+" created</h3>";
				buildDbNav();
				dojo.byId("SelectedDbTitle").innerHTML="";
				clearButtons([]);
			}else{
				dojo.byId("ly_Content").innerHTML=event.status+":Failed";
			}
		},
		error: function(type, data, event) {alert(type+":"+data.message)}
		});
	}
}

function deleteDocument(){
	var confirmText=confirm("Are you sure you want to DELETE this document?");
	if(confirmText){
		dojo.io.bind({
		url:COUCHDBPATH+selectedDb+dojo.byId("currentDocId").innerHTML,
		method:"delete",
		load: function(type, data, event){
			if(event.status=="202"){
				dojo.byId("ly_Content").innerHTML="<h3>Document Deleted</h3>";
				clearButtons(["buttonBack"]);
			}else{
				dojo.byId("ly_Content").innerHTML=event.status+":Failed";
			}
		},
		error: function(type, data, event) {alert(type+":"+data.message)}
		});
	}
}

function viewDocSource(){
	window.open(COUCHDBPATH+selectedDb+dojo.byId("currentDocId").innerHTML)
}

function createNewDocForm(){
	dojo.io.bind({
		url:COUCHDBPATH+"$utils/createDocument.html",
		load:function(type, data, event){
			dojo.byId("ly_Content").innerHTML=data;
			clearButtons(["buttonBack","buttonSaveNewDoc"]);
		},
		error:function(type, data, event){alert(type+":"+data.message);}
	});
}

function saveNewDoc(){
	var tmpXML = "<?xml version='1.0' ?><doc>";
	var counter = 0;
	for(var i=1; i<=7; i++){
		if(dojo.byId("FieldName"+i).value!="" && dojo.byId("FieldValue"+i).value!=""){
			counter += 1;
			tmpXML += "<field name='" + dojo.byId("FieldName" + i).value + "'>"
						 + "<" + dojo.byId("FieldType" + i).value + ">" + dojo.byId("FieldValue" + i).value
						 + "</" + dojo.byId("FieldType" + i).value + "></field>";
		}
	}
	if(counter==0){
		alert("Please fill up at least one row");
		return;
	}
	tmpXML += "</doc>";
	var xml = dojo.dom.createDocumentFromText(tmpXML);
	var ajax = dojo.io.bind({
					 	url: COUCHDBPATH+selectedDb,
            method: "post",
            postContent: xml,
            load:function(type, data, event){
										dojo.byId("ly_Content").innerHTML="<h3>Document Save</h3>";
										clearButtons(["buttonBack"]);
									},
						error:function(type, data, event){alert(type+":"+data.message);},
            headers:{"Content-Type": "application/xml"},
            mimetype: "text/xml",
            transport: "XMLHTTPTransport"
  });
}

function createNewTableForm(){
	dojo.io.bind({
		url:COUCHDBPATH+"$utils/createTable.html",
		load:function(type, data, event){
			dojo.byId("ly_Content").innerHTML=data;
			clearButtons(["buttonBack","buttonSaveNewTable"]);
		},
		error:function(type, data, event){alert(type+":"+data.message);}
	});
}

function saveNewTable(){
	if(dojo.byId("ctfTableId").value==""){
		alert("Please fill in the table id");
		return;
	}else if(dojo.byId("ctfTableName").value==""){
		alert("Please fill in the table name");
		return;
	}else if(dojo.byId("ctfTableSelect").value==""){
		alert("Please fill in the table select statement");
		return;
	}

  var xmlDoc = "<?xml version='1.0' ?>"
  		+ "<doc id='" + dojo.byId("ctfTableId").value + "'>"
			+ "<field name='$table_" + dojo.byId("ctfTableName").value + "'>"
			+ "<text>" + dojo.byId("ctfTableSelect").value + "</text>"
			+ "</field></doc>";
  var xml = dojo.dom.createDocumentFromText(xmlDoc);

  var ajax = dojo.io.bind({
		        url: COUCHDBPATH+selectedDb,
            method: "post",
            postContent: xml,
            load:function(type, data, event){
										dojo.byId("ly_Content").innerHTML="<h3>Table Save</h3>";
										clearButtons(["buttonBack"]);
									},
						error:function(type, data, event){alert(type+":"+data.message);},
            headers:{"Content-Type": "application/xml"},
            mimetype: "text/xml",
            transport: "XMLHTTPTransport"
  });
}